## wav2txt with GUI interface
##
## J. Sluka (JimSluka@InPharmix.com)
## Dec. 2003
##
## GUI interface based on Loft examples
## GUI designed with Loft
##
## Dec 17, 2004 this version does data trimming by 
## omitting all data whose abs value is less then $minVal, 
## which is loaded from the GUI
my $minVal="";
my $keepDwell=20; # Keep every 20th value even if it should be ommited based on the $minVal

## dos command line mode;
##    cd C:\WINDOWS\Desktop\Jim\wav2txt\wav2txt_top_perlbined
##    perl wav2txt_top.pl
##
## To perlbin it!
##    cd C:\WINDOWS\Desktop\Jim\wav2txt\wav2txt_top_perlbined
##    perl -MPerlBin -e"PerlBin->new(qw[ wav2txt_top.pl wav2txt.exe ])"
##
## Required files for distribution
##    wav2txt.exe
##    wav2txt_logo.bmp  the bitmap logo
##    wav2txt_gui.gld   the GUI interface
##    readme.txt
## Useful files for distribution
##    wav2txt_top.pl    the perl source, not required but nice to have
##    piezo2_mod.wav    a sample wav file
## Files NOT required for distribution
##    the entire lib folder 
##    perl58.dll 

use Win32::GUI;
use Win32 ();
use Win32::GUI::Loft;

$version = qq{0.20 (Dec. 2004)};

$includeTime = 1;  # match the initial setting of this option in the GUI

#Note the outmost eval catching fatal errors, reporting 
#them to the user. Try changing the filename and see it 
#fail to open the file.
eval {
	main();
	};
	
Win32::GUI::MessageBox(0, "Error: $@", "wav2txt") if($@);

exit;

#########################################################################
## SUBS
##

## Creates and display the GUI window
sub main {

	my $fileWindow = "wav2txt_gui.gld";  # interface designed/created with GUI Loft
	my $objDesign = Win32::GUI::Loft::Design->newLoad($fileWindow) or
			die("Could not open window file ($fileWindow)");

	my $wav2txt = $objDesign->buildWindow() or die("Could not build window ($fileWindow)");

	#This is the same object as $objDesign. It was placed in the
	#Win32::GUI:Loft::design hash by the buildWindow() method.
	my $design = $Win32::GUI::Loft::design{wav2txt};
	
	#Put the help / program description into its scroll box
	$wav2txt->tfDescription->Text(&loadHelp());
	
	# Display the version number
	$wav2txt->lblSignature->Text("J. Sluka, Version: $version");
	
	#We're ready to go!
	$wav2txt->Show();

	Win32::GUI::Dialog();
	Win32::GUI::MessageBox(0, "Error: $@", "wav2txt") if($@);


	return(1);
	}

#Terminate app.
#Note that we use :: to place the sub in package main,
#just in case you are in another package.

#Note how we verify that the window is actually there
#while assigning the shortcut variable $wav2txt.
sub ::wav2txt_Terminate { 
    defined(my $wav2txt = $Win32::GUI::Loft::window{wav2txt}) or return(1);

	#Just exit
	return(-1);
}

#Terminate app.
sub ::btnExit_Click { 
    defined(my $wav2txt = $Win32::GUI::Loft::window{wav2txt}) or return(1);
	#Win32::GUI::MessageBox(0,'exiting','wav2txt');
	#Just exit
	return(-1);
}

# Browse for file
sub ::btnBrowse_Click {
	defined(my $wav2txt = $Win32::GUI::Loft::window{wav2txt}) or return(1);
	
	#Win32::GUI::MessageBox(0,'Browse','wav2txt');
	# set the global variable $wavFileName
	$wavFileName = GUI::GetOpenFileName(
           -title  => "File Browser",
           -file   => "\0" . " " x 256,
           -filter => [
                "Wav files (*.wav)" => "*.wav",  
                "All files", "*.*",
                 ],
        	);
	
	# create the output file name
	$outFile = $wavFileName;  # $outFile is global
	$outFile =~ s/\.wav$//;
	$outFile .= ".txt";
	
	# Update the in and out file boxes in the GUI
	$wav2txt->tf_InFileName->Text("$wavFileName");
	$wav2txt->txb_outFile_Name->Text("$outFile");
	# Enable the "convert the file" button
	$wav2txt->btn_runButton->Enable(1);
	
	return (1);
}	

# Include time option checkbox
sub ::chb_include_times_chk_Click{
	defined(my $wav2txt = $Win32::GUI::Loft::window{wav2txt}) or return(1);
	
	# should set the $includeTime Variable
	if ($includeTime) {
		undef $includeTime;
		#Win32::GUI::MessageBox(0,'Don\'t include times','wav2txt');
	} else {
		$includeTime = 1;
		#Win32::GUI::MessageBox(0,'include times','wav2txt');
	}
	return (1);
}

# Process the file (crunch the numbers)
sub ::btn_runButton_Click {
	defined(my $wav2txt = $Win32::GUI::Loft::window{wav2txt}) or return(1);
	
	# change the status message in sbStatusBar1
	$wav2txt->sbStatusBar1->Text("Converting wav file ...");	
		
	# get the input and output file names from the GUI since they 
	# may have been changed by the user
	$wavFileName = $wav2txt->tf_InFileName->Text;
	$outFile     = $wav2txt->txb_outFile_Name->Text;
	$outFile2 = $outFile . "2";
	
	# get the trim value from the GUI
	$minVal = $wav2txt->TrimValue->Text;
	#Win32::GUI::MessageBox(0,"Tim Value=$minVal",'wav2txt');
	
	# user can use a wild card in the file name, get all wav files in the current dir
	my $inDir = "";   my @allFiles;   my @allFiles2;   my @allOutFiles;
	if ($wavFileName =~ /\*/) {
		$inDir = $wavFileName;
		$inDir =~ s/^(.*)\\(.*)?$/$1\\/;
		opendir THISDIR,$inDir or die "problem getting directory listing for wildcards!: $!";
			@allFiles = grep /\.wav$/, readdir THISDIR;
		closedir THISDIR;
		# add the path back to the input file names and
		# create array of output file names (the directory path may be different)
		my $outPath = $outFile;
		$outPath =~ s/^(.*)\\(.*)?$/$1\\/;
		for ($iii=0; $iii <= $#allFiles; $iii++) {
			my $fileRoot = $allFiles[$iii];
			$fileRoot =~ s/([^.]*)\..*/$1/;
			$allFiles[$iii] = "$inDir" . $allFiles[$iii];
			$allOutFiles[$iii]  = $outPath . $fileRoot . ".txt";
			$allOutFiles2[$iii] = $outPath . $fileRoot . ".txt2";
		} 
		$wav2txt->txb_outFile_Name->Text("$outPath");
	} else {  # process a single file
		push @allFiles,$wavFileName;
		push @allOutFiles,$outFile;	
		push @allOutFiles2,$outFile2;
	}
	
	# process the file(s)
	for (my $fileIn=0; $fileIn <= $#allFiles; $fileIn++) {
		$convertStatus = &convertWavFile ("$allFiles[$fileIn]", "$allOutFiles[$fileIn]", "$allOutFiles2[$fileIn]", $includeTime);
		$convertStatus += 0;
	}
	$wav2txt->sbStatusBar1->Text("Ready");
	return (1);
}

####################################################
# The help / description text

sub loadHelp {

my $helpText = qq{Wav2Txt converts a .wav audio file into a tab delimited text file.\r\n\r\n};
$helpText .= qq{Click on the "Browse" button to locate a wav file to convert.\r\n};
$helpText .= qq{By default the output file will be placed in the same directory\r\n};
$helpText .= qq{   as the input file and will have a .txt extension.\r\n};
$helpText .= qq{You may change the output path and/or file name in \r\n};
$helpText .= qq{   the "Out:" box.\r\n\r\n};
$helpText .= qq{If the input file name is set to *.wav then all wav files in the\r\n};
$helpText .= qq{   input directory will be converted.\r\n\r\n}; 
$helpText .= qq{If you uncheck the "include time column in output" then the\r\n};
$helpText .= qq{   output file will not include the time values column.\r\n\r\n};

$helpText .= qq{The "Trim Value" trims the output of most data lines that have\r\n};
$helpText .= qq{   |amplitudes| less than the Trim Value. A second output file,\r\n};
$helpText .= qq{   with a .txt2 extension, is created that does not include these\r\n};
$helpText .= qq{   trimmed datapoints. Only every 20th trimmed data point is\r\n};
$helpText .= qq{   included in the .txt2 file. Every data point greater than the\r\n};
$helpText .= qq{   "Trim Value" is always included. This significantly reduces\r\n};
$helpText .= qq{   the size of the outputfile, without losing any important data,\r\n};
$helpText .= qq{   and makes it easier to graph the data. The .txt2 file always\r\n};
$helpText .= qq{   includes the time column.\r\n};
$helpText .= qq{};
$helpText .= qq{};
$helpText .= qq{\r\n};

$helpText .= qq{Click "Convert the file" button to start the conversion.\r\n\r\n};
$helpText .= qq{Script written in Perl.\r\n};
$helpText .= qq{Graphical user interface implemented with Win32::GUI\r\n};
$helpText .= qq{GUI designed using Loft\r\n};
$helpText .= qq{Perl converted to stand-alone executable with perl2exe.\r\n\r\n};
$helpText .= qq{James Sluka (JSluka\@InPharmix.com)\r\n};
$helpText .= qq{Version $version\r\n};

	return($helpText);
} 

####################################################
# Wav conversion routines
# Convert a .wav file to text
#
# useage $convertStatus = &convertWavFile ($inFile, $outFile, $outFile2(trimmed), $includeTime)


sub convertWavFile {
	
	#$inFile = qq{c:/windows/desktop/piezo2_edit.wav};
	my $inFile = shift;
	my $outFile = shift;
	my $outFile2 = shift;
	my $include_Time = shift;
	my $shortFileName = $inFile;
	$shortFileName =~ s/^.*\\//;  # remove the path
	
	if (open (INFILE, "< $inFile") ) {
	#	Win32::GUI::MessageBox(0,"Open input file $inFile OK\noutFile=$outFile",'wav2txt');
	} else {
		Win32::GUI::MessageBox(0,"Couldn't open input file $inFile",'wav2txt');
		return(-1);
	}
	binmode INFILE;

	################################################
	# get the first 256 bytes to extract the header info
	#print qq{fetching } . read (INFILE, $header, 256) . qq{ bytes.\n};
	read (INFILE, $header, 256);

	#vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
	#print "\n\n$inFile\n";
	#my @stuff = split //,$header;
	#for ($iii=0; $iii <= 255; $iii=$iii+8) {
	#	print qq{$iii\t};
	#	for ($jjj=0; $jjj<=7; $jjj++) {
	#		if ($stuff[$iii+$jjj] =~ /\w/) {
	#			print qq{$stuff[$iii+$jjj]  };
	#		} else {
	#			print "x" . unpack "H2",$stuff[$iii+$jjj];
	#		}
	#		print " ";
	#		if ($jjj == 3) {print "  ";}
	#	}
	#	print "  " . ($iii+$jjj-1) . "\n";
	#}
	#^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
	
	################################################
	# split the header up into the various chunks ("fmt_" vs. "fmt "?)
	$header =~ /^(RIFF(.*?)WAVE)(fmt.(.*?))(fact(.*?))?(data(....))/si;

	$RIFF = $1;
	$fmt  = $3;
	$fact = $5;
	$data = $7;
	@RIFF = split //,$RIFF;
	@fmt  = split //,$fmt;
	if (defined $fact) {@fact = split //,$fact;}
	@data = split //,$data;
	
	################################################
	#RIFF Chunk (12 bytes in length total)
	#Byte Number
	#0 - 3	"RIFF" (ASCII Characters)	
	#4 - 7	Total Length Of Package To Follow (Binary, little endian)
	#8 - 11	"WAVE" (ASCII Characters)
#	print qq{\nRIFF Chunk\n};
#	for ($iii=0; $iii <= $#RIFF; $iii++) {
#		print qq{$iii\t};
#		if ($RIFF[$iii] =~ /\w/) {
#			print qq{$RIFF[$iii]};
#		} else {
#			print "x" . unpack "H2",$RIFF[$iii];
#		}
#		print "\n";
#	}

	for ($iii=4; $iii <= 7; $iii++) {
		$RIFF[$iii] = hex unpack "H2",$RIFF[$iii];
	}
	$totL = $RIFF[4] + 256*$RIFF[5] + 256*256*$RIFF[6] + 256*256*256*$RIFF[7];
#	print "totL=$totL=\n";
	
	########################################################
	#FORMAT Chunk (24 bytes in length total)
	#Byte Number
	#0 - 3		"fmt_" (ASCII Characters)
	#4 - 7		Length Of FORMAT Chunk (Binary, always 0x10?)
	#8 - 9		Always 0x01
	#10 - 11	Channel Numbers (Always 0x01=Mono, 0x02=Stereo)
	#12 - 15	Sample Rate (Binary, in Hz)
	#16 - 19	Bytes Per Second
	#20 - 21	Bytes Per Sample: 1=8 bit Mono, 2=8 bit Stereo or 16 bit Mono, 4=16 bit Stereo
	#22 - 23	Bits Per Sample
	#24 - 25	unk?			jps
	
#	print qq{\nFORMAT Chunk\n};
#	for ($iii=0; $iii <= $#fmt; $iii++) {
#		print qq{$iii\t};
#		if ($fmt[$iii] =~ /\w/) {
#			print qq{$fmt[$iii]};
#		} else {
#			print "x" . unpack "H2",$fmt[$iii];
#		}
#		print "\n";
#	}
	
	for ($iii=10; $iii <= 11; $iii++) {$fmt[$iii] = unpack "H2",$fmt[$iii];}
	$channels = $fmt[10] + 256*$fmt[11];
	#print qq{channels =$channels=\n};

	for ($iii=12; $iii <= 15; $iii++) {$fmt[$iii] = hex unpack "H2",$fmt[$iii];}
	$sampleRate = $fmt[12] + 256*$fmt[13] + 256*256*$fmt[14] + 256*256*256*$fmt[15];
	#print qq{sampleRate =$sampleRate=\n};
	
	for ($iii=16; $iii <= 19; $iii++) {$fmt[$iii] = hex unpack "H2",$fmt[$iii];}
	$bytesPerSecond = $fmt[16] + 256*$fmt[17] + 256*256*$fmt[18] + 256*256*256*$fmt[19];
	#print qq{bytesPerSecond =$bytesPerSecond=\n};
	
	for ($iii=20; $iii <= 21; $iii++) {$fmt[$iii] = hex unpack "H2",$fmt[$iii];}
	$bytesPerSample = $fmt[20] + 256*$fmt[21];
	#print qq{bytesPerSample =$bytesPerSample=\n};

	for ($iii=22; $iii <= 23; $iii++) {$fmt[$iii] = hex unpack "H2",$fmt[$iii];}
	$bitsPerSample = $fmt[22] + 256*$fmt[23];
	#print qq{bitsPerSample =$bitsPerSample=\n};
	
	for ($iii=24; $iii <= 25; $iii++) {$fmt[$iii] = hex unpack "H2",$fmt[$iii];}
	$unk = $fmt[24] + 256*$fmt[25];
	#print qq{unk =$unk=\n};
		
	########################################################
	#FACT Chunk (variable number of bytes)
	#Byte Number
	#0 - 3	"fact"			jps
	#4 - 7	bytes in "fact"	jps
	#8 - x 	the "fact"		jps
#	print qq{\nFACT Chunk\n};
#	for ($iii=0; $iii <= $#fact; $iii++) {
#		print qq{$iii\t};
#		if ($fact[$iii] =~ /\w/) {
#			print qq{$fact[$iii]};
#		} else {
#			print "x" . unpack "H2",$fact[$iii];
#		}
#		print "\n";
#	}
	
	$factSize = hexUnpack($fact[4],$fact[5],$fact[6],$fact[7]);
#	print qq{factSize =$factSize=\n};

	$theFact = hexUnpack($fact[8],$fact[9],$fact[10],$fact[11]);
#	print "theFact=$theFact=\n";
	
	######################################################
	# DATA Chunk
	#Byte Number
	#0 - 3	"data" (ASCII Characters)
	#4 - 7	Length Of Data To Follow
	#8 - end	Data (Samples)
#	print qq{\nDATA Chunk\n};
#	for ($iii=0; $iii <= $#data; $iii++) {
#		print qq{$iii\t};
#		if ($data[$iii] =~ /\w/) {
#			print qq{$data[$iii]};
#		} else {
#			print "x" . unpack "H2",$data[$iii];
#		}
#		print "\n";
#	}
	
	for ($iii=4; $iii <= 7; $iii++) {$data[$iii] = hex unpack "H2",$data[$iii];}
	$LOData = $data[4] + 256*$data[5] + 256*256*$data[6] + 256*256*256*$data[7];
	#print qq{LOData =$LOData=\n};
	
	######################################################
	# data offset (# bytes in header)
	$header =~ /(^.*?data....)/s;
	$offset = length($1);
	#print qq{\noffset=$offset\n};
		
	######################################################
	# Set the parameters etc in the GUI for this file
	my $wav2txt = $Win32::GUI::Loft::window{wav2txt};
	$wav2txt->lblInfoTitle->Enable(1);
	$wav2txt->lblInfoTitleV->Text($shortFileName);
	$wav2txt->lblChannels->Enable(1);
	$wav2txt->lblChannelsV->Text($channels);
	$wav2txt->lblSampleRate->Enable(1);
	$wav2txt->lblSampleRateV->Text($sampleRate);
	$wav2txt->lblBytesPerSecond->Enable(1);
	$wav2txt->lblBytesPerSecondV->Text($bytesPerSecond);
	$wav2txt->lblBytesPerSample->Enable(1);
	$wav2txt->lblBytesPerSampleV->Text($bytesPerSample);
	$wav2txt->lblBitsPerSample->Enable(1);
	$wav2txt->lblBitsPerSampleV->Text($bitsPerSample);
	$wav2txt->lblUnknown->Enable(1);
	$wav2txt->lblUnknownV->Text($unk);
	$wav2txt->lblTotalLength->Enable(1);
	$wav2txt->lblTotalLengthV->Text($totL);
	$wav2txt->lblFactSize->Enable(1);
	$wav2txt->lblFactSizeV->Text($factSize);
	$wav2txt->lblTheFact->Enable(1);
	$wav2txt->lblTheFactV->Text($theFact);
	$wav2txt->lblLoData->Enable(1);
	$wav2txt->lblLoDataV->Text($LOData);
	$wav2txt->lblOffset->Enable(1);
	$wav2txt->lblOffsetV->Text($offset);

	######################################################
	# Rewind the file
	close INFILE;
	open (INFILE, "< $inFile") or die "couldn't open input file the second time ==$inFile==, $!";
	binmode INFILE;
	# skip the header
	#print qq{Rewinding, skipping } . read (INFILE, $header, $offset) . " characters.\n";
	read (INFILE, $header, $offset);
	
	######################################################
	# create output file
	# first check to see if a file with the same name already exists
	
	if (-s $outFile) {
		my $theResp = Win32::MsgBox("Output file $outFile already exists, overwrite?",(1+16),"wav2txt");
		if ($theResp ne 1) {
			return -1;
		}
	}
	if (-s $outFile2) {
		my $theResp = Win32::MsgBox("Output file $outFile2 already exists, overwrite?",(1+16),"wav2txt");
		if ($theResp ne 1) {
			return -1;
		}
	}

	
	open  OUTFILE, ">$outFile" or die "Couldn't open output file 1 =$outFile=; $!";
	print OUTFILE "Created from:\t$inFile\n";
	print OUTFILE "Channels:\t$channels\n";
	print OUTFILE "Sample Rate:\t$sampleRate\n";
	print OUTFILE "Bytes per Second:\t$bytesPerSecond\n";
	print OUTFILE "Bytes Per Sample:\t$bytesPerSample\n";
	print OUTFILE "Bits Per Sample:\t$bitsPerSample\n";
	print OUTFILE "Total Samples:\t$LOData\n"; 
	print OUTFILE "\n";

	open  OUTFILE2, ">$outFile2" or die "Couldn't open output file 2 =$outFile2=; $!";
	print OUTFILE2 "Created from:\t$inFile\t\tTRIMMED at +/- $minVal\n";
	print OUTFILE2 "Channels:\t$channels\n";
	print OUTFILE2 "Sample Rate:\t$sampleRate\n";
	print OUTFILE2 "Bytes per Second:\t$bytesPerSecond\n";
	print OUTFILE2 "Bytes Per Sample:\t$bytesPerSample\n";
	print OUTFILE2 "Bits Per Sample:\t$bitsPerSample\n";
	print OUTFILE2 "Total Samples:\t$LOData\n"; 
	print OUTFILE2 "\n";

	if ($include_Time) {
		print OUTFILE  "Time(sec)\tChannel 1";
		print OUTFILE2 "Time(sec)\tChannel 1";
	} else {
		print OUTFILE  "Channel 1";
		print OUTFILE2 "Channel 1";
	}
	if ($channels == 2) {
		print OUTFILE  "\tChannel 2\n";
		print OUTFILE2 "\tChannel 2\n";
	} else {
		print OUTFILE  "\n";
		print OUTFILE2 "\n";
	}

	######################################################
	#Data
	#print qq{\nDATA ...\n};
	my $lastStatusVal = -1;
	my $lastAval = "";   my $lastAtime="";   my $lastAok="no";
	my $lastBval = "";                       my $lastBok="no";
	my $skippedCount = 0;  # $keepDwell=20 defined at top
	my ($aVa, $aVb, $aV1, $aV2, $aV3, $aV4, $nV);
	
	my $bytesPerLine = $bytesPerSample*$channels;  # bytes per time slice
	for ($iii=0; $iii < ($LOData*$channels); $iii += $bytesPerLine ) {

		$nV = read (INFILE, $aV, 1);
		$aV1 = hex unpack "H2",$aV;
		if ($bytesPerLine > 1) {
			$nV = read (INFILE, $aV, 1);
			$aV2 = hex unpack "H2",$aV;
		}
		if ($bytesPerLine > 2) {
			$nV = read (INFILE, $aV, 1);
			$aV3 = hex unpack "H2",$aV;
		};
		if ($bytesPerLine > 3) {
			$nV = read (INFILE, $aV, 1);
			$aV4 = hex unpack "H2",$aV;
		};

		if ($channels == 1) {  # mono
			if ($bytesPerSample == 1) {  # mono, 8bit
				$aVa = $aV1 - 128;
			} else { # $bytesPerSample == 2,  mono, 16bit
				if ($aV2 >= 128) {
					$aVa = -(255-$aV2)*256 + $aV1;
				} else {
					$aVa = 256*$aV2 + $aV1;
				}
			}
		} else {               # stereo
			if ($bytesPerSample == 2) {  # stereo, 8bit
				$aVa = $aV1 - 128;
				$aVb = $aV2 - 128;
			} else { # $bytesPerSample == 4,  stereo, 16bit
				if ($aV2 >= 128) {
					$aVa = -(255-$aV2)*256 + $aV1;
				} else {
					$aVa = 256*$aV2 + $aV1;
				}
				if ($aV4 >= 128) {
					$aVb = -(255-$aV4)*256 + $aV3;
				} else {
					$aVb = 256*$aV4 + $aV3;
				}
			}
		}
		
		# output to the regular file (OUTFILE)
		if ($include_Time) {
			printf OUTFILE "%10.8f\t", $iii/$sampleRate/$bytesPerLine; 
		}
		print OUTFILE "$aVa";
		if ($channels == 2) {
			print OUTFILE "\t$aVb";
		}
		print OUTFILE "\n";
		
		# output to the trimmed file (OUTFILE2)
		# OUTFILE2 always get the time column
		# $skippedCount, $keepDwell
		if ( (abs($aVa) >= $minVal) || (abs($aVb) >= $minVal) ) {  # greater than minValue
			####
			# force the last trimmed point before kept data to output
			if ($lastAok eq "no") { 
				printf OUTFILE2 "%10.8f\t", $lastAtime;
				print  OUTFILE2 "$lastAval";
				#print OUTFILE2 "\tleading forced";
				if ($channels == 2) {print OUTFILE2 "\t$lastBval";}
				print OUTFILE2 "\n";		
			}
			####
			printf OUTFILE2 "%10.8f\t", $iii/$sampleRate/$bytesPerLine;
			print  OUTFILE2 "$aVa";
			if ($channels == 2) {print OUTFILE2 "\t$aVb";}
			print OUTFILE2 "\n";
			$lastAok   = "yes";
			$skippedCount = 0;
		# force every 20th point that would normally be skipped
		} elsif ( ( ($skippedCount % $keepDwell) == 0 ) && ($skippedCount > 0) ) {
			printf OUTFILE2 "%10.8f\t", $iii/$sampleRate/$bytesPerLine;
			print  OUTFILE2 "$aVa";
			if ($channels == 2) {print OUTFILE2 "\t$aVb";}
			#print OUTFILE2 "\tforced";
			print OUTFILE2 "\n";
			$lastAok   = "no";	
			$skippedCount = 0;
		# force the first trimmed point after kept data to output
		} elsif ($lastAok eq "yes") { 
			printf OUTFILE2 "%10.8f\t", $iii/$sampleRate/$bytesPerLine;
			print  OUTFILE2 "$aVa";
			#print OUTFILE2 "\ttrailing forced";
			if ($channels == 2) {print OUTFILE2 "\t$aVb";}
			print OUTFILE2 "\n";
			$lastAok   = "no";	
		} else {
			$lastAok   = "no";
			$skippedCount ++;	
		}
		$lastAval  = $aVa;
		$lastAtime = $iii/$sampleRate/$bytesPerLine;
		$lastBval  = $aVb;
			
		# update the GUI
		my $statusVal = sprintf "%3d",($iii/$LOData*100 + 0.5);
		if ($statusVal ne $lastStatusVal) {
			$wav2txt->sbStatusBar1->Text("Converting $shortFileName file, $statusVal%");
			$lastStatusVal = $statusVal;
		} 
	}
	
	close INFILE;
	close OUTFILE;
	close OUTFILE2;
	
	return(1);
}  # end sub convertWavFile

#######################################################
sub hexUnpack {
	# $theFact = hexUnpack($fact[8],$fact[9],$fact[10],$fact[11]);
	
	@theData = @_;
	my $theVal = 0;
	
	my $exPon = -1;
	for ($iii=0; $iii <= $#theData; $iii++) {
		$exPon ++;
		my $aVal = hex unpack "H2",$theData[$iii];
		my $aInc = $aVal*256**$exPon;
		$theVal += $aInc;
		#print "iii=$iii=\texPon=$exPon=\taVal=$aVal=\ttheVal=$theVal=\n";
	}
	
	return ($theVal);
}  # end sub hexUnpack

1;
